<?php

namespace Drupal\migrate_wizard\Plugin\FieldTypeMW;

use Drupal\field\Entity\FieldStorageConfig;
use Drupal\migrate_wizard\FieldTypeMWBase;

/**
 * Provides 'generic_text_long' field type.
 *
 * @FieldType(
 *   id = "generic_text_long",
 * )
 */
class FieldTypeMWGenericTextLong extends FieldTypeMWBase {

  public static function getMigrationConfig(&$wildcard_yml_config, $origin_field, $data_field, $type_source, $config, $entityFieldManager, $fieldTypeMWManager, $field_collection = NULL) {

    $origin_field_format = $origin_field . '_format';
    $origin_field_value = $origin_field . '_value';
    if ($origin_field == 'description' && $wildcard_yml_config["source"]["origin_type"] == 'taxonomy_term') {
      $origin_field_format = 'format';
      $origin_field_value = 'description';

    }

    $array_config['pseudo_'.$data_field['destiny'] . '_format'][] = [
      'source' => $origin_field_format,
      'plugin' => 'explode',
      'method' => 'process',
      'delimiter' => '||',
    ];
    $array_config['pseudo_'.$data_field['destiny'] . '_format'][] = [
      'plugin' => 'static_map',
      'map' => $config->get('origin_formats'),
      'default_value' => 'full_html',
    ];

    $array_config['pseudo_'.$data_field['destiny'] . '_value'][] = [
      'plugin' => 'skip_on_empty',
      'source' => $origin_field_value,
      'method' => 'process',
    ];
    $array_config['pseudo_'.$data_field['destiny'] . '_value'][] = [
      'plugin' => 'explode',
      'method' => 'process',
      'delimiter' => '||',
    ];
    $array_config['pseudo_'.$data_field['destiny']][] = [
      'plugin' => 'skip_on_empty',
      'source' => ['@pseudo_'.$data_field['destiny'] . '_value', '@pseudo_'.$data_field['destiny'] . '_format'],
      'method' => 'process',
    ];

    $array_config['pseudo_'.$data_field['destiny']][] = [
      'plugin' => 'text_long_with_format',
    ];

    $array_config[$data_field['destiny']][] = [
      'plugin' => 'skip_on_empty',
      'source' => '@pseudo_'.$data_field['destiny'],
      'method' => 'process'
    ];
    $array_config[$data_field['destiny']][] = [
      'plugin' => 'sub_process',
      'method' => 'process',
      'process' => ['value' => 'value', 'format' => 'format'],
    ];

    if(!$field_collection){
      $wildcard_yml_config['process'] = array_merge($wildcard_yml_config['process'], $array_config);
    }else{
      foreach($array_config[$data_field['destiny']] as $value){
        $wildcard_yml_config['process'] = parent::insertBeforeKey($wildcard_yml_config['process'], $field_collection, $data_field['destiny'], $value);
      }
    }
  }

  public function addQueryByType($alias, $table, $data_field, $wildcard, $translation, $field_collection = NULL) {
    if($alias != 'description' || $wildcard->getValue('entityType') != 'taxonomy_term'){
      $base_field = str_replace('field_data_', '', $table);

      if($field_collection == NULL){
        $wildcard->query->leftJoin($table, $alias, "{$wildcard->getValue('leftJoinTable')} = $alias.entity_id AND $alias.entity_type = '{$wildcard->getValue('entityType')}' AND $alias.bundle = '{$wildcard->getValue('bundle')}'");
      }else{
        $wildcard->query->leftJoin($table, $alias, "{$field_collection['left_join_table']} = $alias.entity_id AND $alias.entity_type = 'field_collection_item' AND $alias.bundle = '{$field_collection['bundle']}'");
      }
      $wildcard->query->addExpression("GROUP_CONCAT(DISTINCT COALESCE($alias.{$base_field}_value, '') ORDER BY $alias.delta ASC  SEPARATOR '||')", $base_field.'_value', []);
      $wildcard->query->addExpression("GROUP_CONCAT(DISTINCT COALESCE($alias.{$base_field}_format, '') ORDER BY $alias.delta ASC  SEPARATOR '||')", $base_field.'_format', []);
    }
  }
}
